"""
source = Sim('autosave/evo01-2.sim')
for i, group in zip(range(5), source.parse()):
     print group
     print '-----------'
globalTree=None
for bot in source.parse():
	bot.reverse()
	if not globalTree:
		globalTree = Tree(bot)
		pass
	else:
		newTree=Tree(bot)
		globalTree = globalTree.join(Tree(bot))
print globalTree, '\n'
"""
import re, operator

path = '', r'./Saves/', r'./Autosave/'

class InvalidJoin(Exception):
    pass

class Sim(object):
    """
    Represents a DB sim file. 
    """
    def __init__(self, filename):
        for s in path:
            try:
                open(s+filename,'r+b')
                self.filename = s + filename
                break
            except IOError:
                pass
        if hasattr(self,'filename'):
            self.genomeList={}
            self.phenoList={}
            print "File", self.filename, "found"
        else:
            print "No file found matching", filename        

    def parse(self): 
        '''
        Generator 
        '''
        f = open(self.filename,'rb')
        print 'Reading file'
        eof = False
        lines = []
        r2 = re.compile(r'(Copy Error|Point Mutation|Minor Deletion|Major Deletion|Insertion|Reversal|Delta mutations).*\r\n$', re.DOTALL)
        while not eof:
            stop = False
            line = f.readline()
            if line == '':
                eof = True
            else:
                m = r2.search(line)
                if m:
                    lines.append( m.group()[:-2])
                elif lines:
                    yield lines
                    lines = []

    def makeTree(self):
        tree=None
        for bot in self.parse():
            bot.reverse()
            if not tree:
                tree = Tree(bot)
                pass
            else:
                newTree=Tree(bot)
                tree = tree.join(Tree(bot))
        tree.sort()
        return tree
        
class History(list):
    def __init__ (self, record):
        list.__init__(self)
        self = [item for item in reversed(record)]

class Tree(object):
    def __init__ (self, history = None, parent = None, count = 1):
        self.parent = parent
        self.children = []
        self.ownCount = count         
        if history:
            self.ownHist = history[:]  
        else:
            self.ownHist = []

    def __str__(self):
        return self.strTree()

    def strTree(self, init = ''):
        myHist = self.strHistory( init)
        if init and self is self.parent.children[-1]:
            init = init[:-1] + ' '
        temp = [n.strTree(init + '|') for n in self.children]
        temp.insert(0, myHist)
        return '\n'.join(temp)

    def strHistory(self, init = ''):
        return init + '+<%d mutations> (%d / %d ) genomes.' \
               % (len(self.ownHist), self.ownCount, self.inclusiveCount())

    def inclusiveCount(self):
        return self.ownCount + sum(n.inclusiveCount() for n in self.children)

    def join(self, other):
        if self.parent != other.parent: 
            raise InvalidJoin
        if self is other:
            raise InvalidJoin
        if not (self.ownHist and other.ownHist):
            raise InvalidJoin
        commonHist=[]
        for mut1, mut2 in zip(self.ownHist,other.ownHist):
            if mut1 == mut2:
                commonHist.append(mut1)
                self.ownHist.pop(0)
                other.ownHist.pop(0)
            else:
                if not commonHist:
                    return None
                break
        if self.ownHist:
            ancestor = Tree(commonHist, self.parent, 0)
            self.parent = ancestor
            if other.ownHist:
                other.parent = ancestor
                ancestor.children = [self, other]
            else:
                ancestor.children = [self]
                ancestor.ownCount += other.ownCount
            return self.parent
        else:
            self.ownHist = commonHist[:]
            if other.ownHist:
                other.parent = self
                for i in range(len(self.children)):
                    newChild = self.children[i].join(other)
                    if newChild:
                        self.children[i] = newChild
                        return self
                self.children.append(other) 
            else:
                self.ownCount += other.ownCount
            return self

    def sort(self, reverse = False):
        for n in self.children:
            n.sort(reverse)
        self.children.sort(None, Tree.inclusiveCount, reverse)

def report(filename):
    source = Sim(filename)
    tree = source.makeTree()
    return str(tree)

if __name__ == "__main__":
    from optparse import OptionParser
    import sys
    parser = OptionParser(usage = "usage: %prog [opts] <INPUT>.sim")
    parser.add_option("-o", "--output", dest="output",
                      help="write report to FILE", metavar="FILE")
    (options, args) = parser.parse_args()
    if args:
        print report(args[0])
    else:
        parser.print_help()
   
    sys.exit()
        
    
